@@ -1,7 +1,11 @@
 from django.test import TestCase
 
-from nautobot.circuits.choices import *
-from nautobot.circuits.filters import *
+from nautobot.circuits.filters import (
+    CircuitFilterSet,
+    CircuitTerminationFilterSet,
+    CircuitTypeFilterSet,
+    ProviderFilterSet,
+)
 from nautobot.circuits.models import Circuit, CircuitTermination, CircuitType, Provider
 from nautobot.dcim.models import Cable, Device, DeviceRole, DeviceType, Interface, Manufacturer, Region, Site
 from nautobot.extras.models import Status
