@@ -1,5 +1,6 @@
 import django_filters
 from django.contrib.auth import get_user_model
+from django.db.models import Q
 
 from nautobot.extras.filters import (
     CustomFieldModelFilterSet,
@@ -7,6 +8,7 @@ from nautobot.extras.filters import (
     CreatedUpdatedFilterSet,
     StatusModelFilterSetMixin,
 )
+from nautobot.extras.models import SecretsGroup
 from nautobot.tenancy.filters import TenancyFilterSet
 from nautobot.tenancy.models import Tenant
 from nautobot.utilities.choices import ColorChoices
@@ -14,14 +16,21 @@ from nautobot.utilities.filters import (
     BaseFilterSet,
     MultiValueCharFilter,
     MultiValueMACAddressFilter,
-    MultiValueNumberFilter,
     NameSlugSearchFilterSet,
     TagFilter,
     TreeNodeMultipleChoiceFilter,
 )
 from nautobot.virtualization.models import Cluster
-from .choices import *
-from .constants import *
+from .choices import (
+    CableTypeChoices,
+    ConsolePortTypeChoices,
+    InterfaceTypeChoices,
+    PowerOutletTypeChoices,
+    PowerPortTypeChoices,
+    RackTypeChoices,
+    RackWidthChoices,
+)
+from .constants import NONCONNECTABLE_IFACE_TYPES, VIRTUAL_IFACE_TYPES, WIRELESS_IFACE_TYPES
 from .models import (
     Cable,
     ConsolePort,
@@ -327,7 +336,7 @@ class RackFilterSet(
         )
 
 
-class RackReservationFilterSet(BaseFilterSet, TenancyFilterSet):
+class RackReservationFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet, TenancyFilterSet):
     q = django_filters.CharFilter(
         method="search",
         label="Search",
@@ -682,6 +691,17 @@ class DeviceFilterSet(
         method="_has_primary_ip",
         label="Has a primary IP",
     )
+    secrets_group_id = django_filters.ModelMultipleChoiceFilter(
+        field_name="secrets_group",
+        queryset=SecretsGroup.objects.all(),
+        label="Secrets group (ID)",
+    )
+    secrets_group = django_filters.ModelMultipleChoiceFilter(
+        field_name="secrets_group__slug",
+        queryset=SecretsGroup.objects.all(),
+        to_field_name="slug",
+        label="Secrets group (slug)",
+    )
     virtual_chassis_id = django_filters.ModelMultipleChoiceFilter(
         field_name="virtual_chassis",
         queryset=VirtualChassis.objects.all(),
@@ -1068,7 +1088,7 @@ class InventoryItemFilterSet(BaseFilterSet, DeviceComponentFilterSet):
         return queryset.filter(qs_filter)
 
 
-class VirtualChassisFilterSet(BaseFilterSet):
+class VirtualChassisFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet):
     q = django_filters.CharFilter(
         method="search",
         label="Search",
@@ -1131,7 +1151,7 @@ class VirtualChassisFilterSet(BaseFilterSet):
         return queryset.filter(qs_filter)
 
 
-class CableFilterSet(StatusModelFilterSetMixin, BaseFilterSet):
+class CableFilterSet(StatusModelFilterSetMixin, BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet):
     q = django_filters.CharFilter(
         method="search",
         label="Search",
@@ -1215,7 +1235,7 @@ class InterfaceConnectionFilterSet(ConnectionFilterSet, BaseFilterSet):
         fields = []
 
 
-class PowerPanelFilterSet(BaseFilterSet):
+class PowerPanelFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet):
     q = django_filters.CharFilter(
         method="search",
         label="Search",
