@@ -16,12 +16,18 @@ import yaml
 
 from nautobot.core.celery import nautobot_task
 from nautobot.dcim.models import Device, DeviceRole, DeviceType, Platform, Region, Site
-from nautobot.extras.choices import LogLevelChoices, JobResultStatusChoices
+from nautobot.extras.choices import (
+    JobResultStatusChoices,
+    LogLevelChoices,
+    SecretsGroupAccessTypeChoices,
+    SecretsGroupSecretTypeChoices,
+)
 from nautobot.extras.models import (
     ConfigContext,
     ConfigContextSchema,
     ExportTemplate,
     GitRepository,
+    JobLogEntry,
     JobResult,
     Tag,
 )
@@ -103,7 +109,7 @@ def pull_git_repository_and_refresh_data(repository_pk, request, job_result_pk):
 
     finally:
         if job_result.status not in JobResultStatusChoices.TERMINAL_STATE_CHOICES:
-            if job_result.data["total"][LogLevelChoices.LOG_FAILURE] > 0:
+            if JobLogEntry.objects.filter(job_result__pk=job_result.pk, log_level=LogLevelChoices.LOG_FAILURE).exists():
                 job_result.set_status(JobResultStatusChoices.STATUS_FAILED)
             else:
                 job_result.set_status(JobResultStatusChoices.STATUS_COMPLETED)
@@ -128,10 +134,38 @@ def ensure_git_repository(repository_record, job_result=None, logger=None, head=
       head (str): Optional Git commit hash to check out instead of pulling branch latest.
     """
 
-    # Inject token into source URL if necessary
+    # Inject username and/or token into source URL if necessary
     from_url = repository_record.remote_url
-    token = repository_record._token
-    user = repository_record.username
+
+    user = None
+    token = None
+    if repository_record.secrets_group:
+        # In addition to ObjectDoesNotExist, get_secret_value() may also raise a SecretError if a secret is mis-defined;
+        # we don't catch that here but leave it up to the caller to handle as part of general exception handling.
+        try:
+            token = repository_record.secrets_group.get_secret_value(
+                SecretsGroupAccessTypeChoices.TYPE_HTTP,
+                SecretsGroupSecretTypeChoices.TYPE_TOKEN,
+                obj=repository_record,
+            )
+        except ObjectDoesNotExist:
+            # No defined secret, fall through to legacy behavior
+            pass
+        try:
+            user = repository_record.secrets_group.get_secret_value(
+                SecretsGroupAccessTypeChoices.TYPE_HTTP,
+                SecretsGroupSecretTypeChoices.TYPE_USERNAME,
+                obj=repository_record,
+            )
+        except ObjectDoesNotExist:
+            # No defined secret, fall through to legacy behavior
+            pass
+
+    if not token and repository_record._token:
+        token = repository_record._token
+    if not user and repository_record.username:
+        user = repository_record.username
+
     if token and token not in from_url:
         # Some git repositories require a user as well as a token.
         if user:
@@ -608,7 +642,7 @@ def update_git_config_context_schemas(repository_record, job_result):
             continue
         job_result.log(
             f"Loading config context schema from `{file_name}`",
-            grouping="config contexts",
+            grouping="config context schemas",
             logger=logger,
         )
         try:
