@@ -1,19 +1,24 @@
 import logging
 from collections import OrderedDict
+
+from django import forms
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
 from django.core.serializers.json import DjangoJSONEncoder
 from django.core.validators import ValidationError
 from django.db import models
+from django.db.models import Q
 
 from nautobot.extras.choices import RelationshipTypeChoices, RelationshipSideChoices
 from nautobot.extras.utils import FeatureQuery
 from nautobot.extras.models import ChangeLoggedModel
+from nautobot.core.fields import AutoSlugField
 from nautobot.core.models import BaseModel
 from nautobot.utilities.utils import get_filterset_for_model
 from nautobot.utilities.forms import (
     DynamicModelChoiceField,
     DynamicModelMultipleChoiceField,
+    widgets,
 )
 from nautobot.utilities.querysets import RestrictedQuerySet
 
@@ -62,6 +67,10 @@ class RelationshipModel(models.Model):
                     <relationship #3>: <queryset #3>,
                     <relationship #4>: <queryset #4>,
                 },
+                "peer": {
+                    <relationship #5>: <queryset #5>,
+                    <relationship #6>: <queryset #6>,
+                },
             }
         """
         src_relationships, dst_relationships = Relationship.objects.get_for_model(self)
@@ -75,6 +84,7 @@ class RelationshipModel(models.Model):
         resp = {
             RelationshipSideChoices.SIDE_SOURCE: OrderedDict(),
             RelationshipSideChoices.SIDE_DESTINATION: OrderedDict(),
+            RelationshipSideChoices.SIDE_PEER: OrderedDict(),
         }
         for side, relationships in sides.items():
             for relationship in relationships:
@@ -94,10 +104,21 @@ class RelationshipModel(models.Model):
 
                 # Construct the queryset to query all RelationshipAssociation for this object and this relationship
                 query_params = {"relationship": relationship}
-                query_params[f"{side}_id"] = self.pk
-                query_params[f"{side}_type"] = content_type
+                if not relationship.symmetric:
+                    # Query for RelationshipAssociations that this object is on the expected side of
+                    query_params[f"{side}_id"] = self.pk
+                    query_params[f"{side}_type"] = content_type
 
-                resp[side][relationship] = RelationshipAssociation.objects.filter(**query_params)
+                    resp[side][relationship] = RelationshipAssociation.objects.filter(**query_params)
+                else:
+                    # Query for RelationshipAssociations involving this object, regardless of side
+                    resp[RelationshipSideChoices.SIDE_PEER][relationship] = RelationshipAssociation.objects.filter(
+                        (
+                            Q(source_id=self.pk, source_type=content_type)
+                            | Q(destination_id=self.pk, destination_type=content_type)
+                        ),
+                        **query_params,
+                    )
 
         return resp
 
@@ -126,6 +147,9 @@ class RelationshipModel(models.Model):
                 "destination": {
                     (same format as source)
                 },
+                "peer": {
+                    (same format as source)
+                },
             }
         """
 
@@ -134,6 +158,7 @@ class RelationshipModel(models.Model):
         resp = {
             RelationshipSideChoices.SIDE_SOURCE: OrderedDict(),
             RelationshipSideChoices.SIDE_DESTINATION: OrderedDict(),
+            RelationshipSideChoices.SIDE_PEER: OrderedDict(),
         }
         for side, relationships in relationships_by_side.items():
             for relationship, queryset in relationships.items():
@@ -142,20 +167,25 @@ class RelationshipModel(models.Model):
 
                 resp[side][relationship] = {
                     "label": relationship.get_label(side),
-                    "peer_type": getattr(relationship, f"{peer_side}_type"),
                     "value": None,
                 }
+                if not relationship.symmetric:
+                    resp[side][relationship]["peer_type"] = getattr(relationship, f"{peer_side}_type")
+                else:
+                    # Symmetric relationship - source_type == destination_type, so it doesn't matter which we choose
+                    resp[side][relationship]["peer_type"] = getattr(relationship, "source_type")
 
                 resp[side][relationship]["has_many"] = relationship.has_many(peer_side)
 
                 if resp[side][relationship]["has_many"]:
                     resp[side][relationship]["queryset"] = queryset
                 else:
-                    cra = queryset.first()
-                    if not cra:
+                    association = queryset.first()
+                    if not association:
                         continue
 
-                    peer = getattr(cra, peer_side)
+                    peer = association.get_peer(self)
+
                     resp[side][relationship]["value"] = peer
                     resp[side][relationship]["url"] = peer.get_absolute_url()
 
@@ -179,12 +209,13 @@ class RelationshipManager(models.Manager.from_queryset(RestrictedQuerySet)):
 class Relationship(BaseModel, ChangeLoggedModel):
 
     name = models.CharField(max_length=100, unique=True, help_text="Internal relationship name")
-    slug = models.SlugField(max_length=100, unique=True)
+    slug = AutoSlugField(populate_from="name")
     description = models.CharField(max_length=200, blank=True)
     type = models.CharField(
         max_length=50,
         choices=RelationshipTypeChoices,
         default=RelationshipTypeChoices.TYPE_MANY_TO_MANY,
+        help_text="Cardinality of this relationship",
     )
 
     #
@@ -196,13 +227,13 @@ class Relationship(BaseModel, ChangeLoggedModel):
         related_name="source_relationships",
         verbose_name="Source Object",
         limit_choices_to=FeatureQuery("relationships"),
-        help_text="The source object to which this relationship applies.",
+        help_text="The source object type to which this relationship applies.",
     )
     source_label = models.CharField(
         max_length=50,
         blank=True,
         verbose_name="Source Label",
-        help_text="Name of the relationship as displayed on the source object.",
+        help_text="Label for related destination objects, as displayed on the source object.",
     )
     source_hidden = models.BooleanField(
         default=False,
@@ -225,13 +256,13 @@ class Relationship(BaseModel, ChangeLoggedModel):
         related_name="destination_relationships",
         verbose_name="Destination Object",
         limit_choices_to=FeatureQuery("relationships"),
-        help_text="The destination object to which this relationship applies.",
+        help_text="The destination object type to which this relationship applies.",
     )
     destination_label = models.CharField(
         max_length=50,
         blank=True,
         verbose_name="Destination Label",
-        help_text="Name of the relationship as displayed on the destination object.",
+        help_text="Label for related source objects, as displayed on the destination object.",
     )
     destination_hidden = models.BooleanField(
         default=False,
@@ -253,6 +284,13 @@ class Relationship(BaseModel, ChangeLoggedModel):
     def __str__(self):
         return self.name.replace("_", " ")
 
+    @property
+    def symmetric(self):
+        return self.type in (
+            RelationshipTypeChoices.TYPE_ONE_TO_ONE_SYMMETRIC,
+            RelationshipTypeChoices.TYPE_MANY_TO_MANY_SYMMETRIC,
+        )
+
     def get_label(self, side):
         """Return the label for a given side, source or destination.
 
@@ -262,13 +300,20 @@ class Relationship(BaseModel, ChangeLoggedModel):
         if side not in VALID_SIDES:
             raise ValueError(f"side value can only be: {','.join(VALID_SIDES)}")
 
+        # Peer "side" implies symmetric relationship, where source and dest are equivalent
+        if side == RelationshipSideChoices.SIDE_PEER:
+            side = RelationshipSideChoices.SIDE_SOURCE
+
         if getattr(self, f"{side}_label"):
             return getattr(self, f"{side}_label")
 
         if side == RelationshipSideChoices.SIDE_SOURCE:
             destination_model = self.destination_type.model_class()
+            if not destination_model:  # perhaps a plugin was uninstalled?
+                return str(self)
             if self.type in (
                 RelationshipTypeChoices.TYPE_MANY_TO_MANY,
+                RelationshipTypeChoices.TYPE_MANY_TO_MANY_SYMMETRIC,
                 RelationshipTypeChoices.TYPE_ONE_TO_MANY,
             ):
                 return destination_model._meta.verbose_name_plural
@@ -277,7 +322,12 @@ class Relationship(BaseModel, ChangeLoggedModel):
 
         elif side == RelationshipSideChoices.SIDE_DESTINATION:
             source_model = self.source_type.model_class()
-            if self.type == RelationshipTypeChoices.TYPE_MANY_TO_MANY:
+            if not source_model:  # perhaps a plugin was uninstalled?
+                return str(self)
+            if self.type in (
+                RelationshipTypeChoices.TYPE_MANY_TO_MANY,
+                RelationshipTypeChoices.TYPE_MANY_TO_MANY_SYMMETRIC,
+            ):
                 return source_model._meta.verbose_name_plural
             else:
                 return source_model._meta.verbose_name
@@ -290,19 +340,17 @@ class Relationship(BaseModel, ChangeLoggedModel):
         if side not in VALID_SIDES:
             raise ValueError(f"side value can only be: {','.join(VALID_SIDES)}")
 
-        if self.type == RelationshipTypeChoices.TYPE_MANY_TO_MANY:
+        if self.type in (
+            RelationshipTypeChoices.TYPE_MANY_TO_MANY,
+            RelationshipTypeChoices.TYPE_MANY_TO_MANY_SYMMETRIC,
+        ):
             return True
 
-        if self.type == RelationshipTypeChoices.TYPE_ONE_TO_ONE:
+        if self.type in (RelationshipTypeChoices.TYPE_ONE_TO_ONE, RelationshipTypeChoices.TYPE_ONE_TO_ONE_SYMMETRIC):
             return False
 
-        if side == RelationshipSideChoices.SIDE_SOURCE and self.type == RelationshipTypeChoices.TYPE_ONE_TO_MANY:
-            return False
-
-        if side == RelationshipSideChoices.SIDE_DESTINATION and self.type == RelationshipTypeChoices.TYPE_ONE_TO_MANY:
-            return True
-
-        return None
+        # ONE_TO_MANY
+        return side == RelationshipSideChoices.SIDE_DESTINATION
 
     def to_form_field(self, side):
         """
@@ -314,18 +362,31 @@ class Relationship(BaseModel, ChangeLoggedModel):
 
         peer_side = RelationshipSideChoices.OPPOSITE[side]
 
-        object_type = getattr(self, f"{peer_side}_type")
-        filters = getattr(self, f"{peer_side}_filter") or {}
+        if peer_side != RelationshipSideChoices.SIDE_PEER:
+            object_type = getattr(self, f"{peer_side}_type")
+            filters = getattr(self, f"{peer_side}_filter") or {}
+        else:
+            # Symmetric relationship - source and dest fields are presumed identical, so just use source
+            object_type = getattr(self, "source_type")
+            filters = getattr(self, "source_filter") or {}
 
-        queryset = object_type.model_class().objects.all()
+        model_class = object_type.model_class()
+        if model_class:
+            queryset = model_class.objects.all()
+        else:  # maybe a relationship to a model that no longer exists, such as a removed plugin?
+            queryset = None
 
         field_class = None
-        if self.has_many(peer_side):
-            field_class = DynamicModelMultipleChoiceField
+        if queryset:
+            if self.has_many(peer_side):
+                field_class = DynamicModelMultipleChoiceField
+            else:
+                field_class = DynamicModelChoiceField
+
+            field = field_class(queryset=queryset, query_params=filters)
         else:
-            field_class = DynamicModelChoiceField
+            field = forms.MultipleChoiceField(widget=widgets.StaticSelect2Multiple)
 
-        field = field_class(queryset=queryset, query_params=filters)
         field.model = self
         field.required = False
         field.label = self.get_label(side)
@@ -336,9 +397,6 @@ class Relationship(BaseModel, ChangeLoggedModel):
 
     def clean(self):
 
-        if self.source_type == self.destination_type:
-            raise ValidationError("Not supported to have the same Objet type for Source and Destination.")
-
         # Check if source and destination filters are valid
         for side in ["source", "destination"]:
             if not getattr(self, f"{side}_filter"):
@@ -346,6 +404,8 @@ class Relationship(BaseModel, ChangeLoggedModel):
 
             filter = getattr(self, f"{side}_filter")
             side_model = getattr(self, f"{side}_type").model_class()
+            if not side_model:  # can happen if for example a plugin providing the model was uninstalled
+                raise ValidationError({f"{side}_type": "Unable to locate model class"})
             model_name = side_model._meta.label
             if not isinstance(filter, dict):
                 raise ValidationError({f"{side}_filter": f"Filter for {model_name} must be a dictionary"})
@@ -372,6 +432,31 @@ class Relationship(BaseModel, ChangeLoggedModel):
             if error_messages:
                 raise ValidationError({f"{side}_filter": error_messages})
 
+        if self.symmetric:
+            # For a symmetric relation, source and destination attributes must be equivalent if specified
+            error_messages = {}
+            if self.source_type != self.destination_type:
+                error_messages["destination_type"] = "Must match source_type for a symmetric relationship"
+            if self.source_label != self.destination_label:
+                if not self.source_label:
+                    self.source_label = self.destination_label
+                elif not self.destination_label:
+                    self.destination_label = self.source_label
+                else:
+                    error_messages["destination_label"] = "Must match source_label for a symmetric relationship"
+            if self.source_hidden != self.destination_hidden:
+                error_messages["destination_hidden"] = "Must match source_hidden for a symmetric relationship"
+            if self.source_filter != self.destination_filter:
+                if not self.source_filter:
+                    self.source_filter = self.destination_filter
+                elif not self.destination_filter:
+                    self.destination_filter = self.source_filter
+                else:
+                    error_messages["destination_filter"] = "Must match source_filter for a symmetric relationship"
+
+            if error_messages:
+                raise ValidationError(error_messages)
+
         # If the model already exist, ensure that it's not possible to modify the source or destination type
         if self.present_in_database:
             nbr_existing_cras = RelationshipAssociation.objects.filter(relationship=self).count()
@@ -416,7 +501,10 @@ class RelationshipAssociation(BaseModel):
         )
 
     def __str__(self):
-        return "{} -> {} - {}".format(self.source, self.destination, self.relationship)
+        if self.relationship.symmetric:
+            return "{} <-> {} - {}".format(self.source, self.destination, self.relationship)
+        else:
+            return "{} -> {} - {}".format(self.source, self.destination, self.relationship)
 
     def get_peer(self, obj):
 
@@ -438,33 +526,73 @@ class RelationshipAssociation(BaseModel):
                 {"destination_type": f"destination_type has a different value than defined in {self.relationship}"}
             )
 
-        # Check if a similar relationship already exist
-        if self.relationship.type != RelationshipTypeChoices.TYPE_MANY_TO_MANY:
+        if self.source_type == self.destination_type and self.source_id == self.destination_id:
+            raise ValidationError({"destination_id": "An object cannot form a RelationshipAssociation with itself"})
 
-            count_dest = RelationshipAssociation.objects.filter(
+        if self.relationship.symmetric:
+            # Check for a "duplicate" record that exists with source and destination swapped
+            if RelationshipAssociation.objects.filter(
+                relationship=self.relationship,
+                destination_id=self.source_id,
+                source_id=self.destination_id,
+            ).exists():
+                raise ValidationError(
+                    {
+                        "__all__": f"A {self.relationship} association already exists between {self.source} and {self.destination}"
+                    }
+                )
+
+        # Check if a similar relationship association already exists in violation of relationship type cardinality
+        if self.relationship.type not in (
+            RelationshipTypeChoices.TYPE_MANY_TO_MANY,
+            RelationshipTypeChoices.TYPE_MANY_TO_MANY_SYMMETRIC,
+        ):
+            # Either one-to-many or one-to-one, in either case don't allow multiple sources to the same destination
+            if RelationshipAssociation.objects.filter(
                 relationship=self.relationship,
                 destination_type=self.destination_type,
                 destination_id=self.destination_id,
-            ).count()
-
-            if count_dest != 0:
+            ).exists():
                 raise ValidationError(
                     {
                         "destination": f"Unable to create more than one {self.relationship} association to {self.destination} (destination)"
                     }
                 )
 
-            if self.relationship.type == RelationshipTypeChoices.TYPE_ONE_TO_ONE:
-
-                count_src = RelationshipAssociation.objects.filter(
+            if self.relationship.type in (
+                RelationshipTypeChoices.TYPE_ONE_TO_ONE,
+                RelationshipTypeChoices.TYPE_ONE_TO_ONE_SYMMETRIC,
+            ):
+                # Don't allow multiple destinations from the same source
+                if RelationshipAssociation.objects.filter(
                     relationship=self.relationship,
                     source_type=self.source_type,
                     source_id=self.source_id,
-                ).count()
-
-                if count_src != 0:
+                ).exists():
                     raise ValidationError(
                         {
                             "source": f"Unable to create more than one {self.relationship} association to {self.source} (source)"
                         }
                     )
+
+            if self.relationship.type == RelationshipTypeChoices.TYPE_ONE_TO_ONE_SYMMETRIC:
+                # Handle the case where the source and destination fields (which are interchangeable for a symmetric
+                # relationship) are swapped around - sneaky!
+                if RelationshipAssociation.objects.filter(
+                    relationship=self.relationship,
+                    destination_id=self.source_id,
+                ).exists():
+                    raise ValidationError(
+                        {
+                            "source": f"Unable to create more than one {self.relationship} association involving {self.source} (peer)"
+                        }
+                    )
+                if RelationshipAssociation.objects.filter(
+                    relationship=self.relationship,
+                    source_id=self.destination_id,
+                ).exists():
+                    raise ValidationError(
+                        {
+                            "destination": f"Unable to create more than one {self.relationship} association involving {self.destination} (peer)"
+                        }
+                    )
