@@ -1,6 +1,7 @@
 import json
 from io import StringIO
 import os
+from unittest import mock
 import uuid
 
 from django.core.exceptions import ObjectDoesNotExist
@@ -10,14 +11,24 @@ from django.core.management.base import CommandError
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
+from django.test.client import RequestFactory
 
 from nautobot.dcim.models import DeviceRole, Site
-from nautobot.extras.choices import JobResultStatusChoices
+from nautobot.extras.choices import JobResultStatusChoices, LogLevelChoices
 from nautobot.extras.jobs import get_job, run_job
-from nautobot.extras.models import FileAttachment, FileProxy, JobResult, Status
+from nautobot.extras.models import FileProxy, JobResult, Status
+from nautobot.extras.models.models import JobLogEntry
 from nautobot.utilities.testing import CeleryTestCase, TestCase
 
 
+# Use the proper swappable User model
+User = get_user_model()
+
+
+# Override the JOB_LOGS to None so that the Log Objects are created in the default database.
+# This change is required as JOB_LOGS is a `fake` database pointed at the default. The django
+# database cleanup will fail and cause tests to fail as this is not a real database.
+@mock.patch("nautobot.extras.models.models.JOB_LOGS", None)
 class JobTest(TestCase):
     """
     Test basic jobs to ensure importing works.
@@ -29,6 +40,14 @@ class JobTest(TestCase):
     def setUpTestData(cls):
         cls.job_content_type = ContentType.objects.get(app_label="extras", model="job")
 
+    def setUp(self):
+        super().setUp()
+
+        # Initialize fake request that will be required to execute Webhooks (in jobs.)
+        self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
+        self.request.id = uuid.uuid4()
+        self.request.user = self.user
+
     def test_job_pass(self):
         """
         Job test with pass result.
@@ -83,7 +102,10 @@ class JobTest(TestCase):
 
             self.assertHTMLEqual(
                 form.as_table(),
-                """<tr><th><label for="id_var2">Var2:</label></th><td>
+                """<tr><th><label for="id_var1">Var1:</label></th><td>
+<input class="form-control form-control" id="id_var1" name="var1" placeholder="None" required type="file">
+<br><span class="helptext">Some file wants to be first</span></td></tr>
+<tr><th><label for="id_var2">Var2:</label></th><td>
 <input class="form-control form-control" id="id_var2" name="var2" placeholder="None" required type="text">
 <br><span class="helptext">Hello</span></td></tr>
 <tr><th><label for="id_var23">Var23:</label></th><td>
@@ -161,9 +183,9 @@ class JobTest(TestCase):
             self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
             self.assertEqual(Site.objects.count(), 0)  # Ensure DB transaction was aborted
             # Also ensure the standard log message about aborting the transaction is *not* present
-            self.assertNotEqual(
-                job_result.data["run"]["log"][-1][-1], "Database changes have been reverted due to error."
-            )
+            run_log = JobLogEntry.objects.filter(grouping="run")
+            for log in run_log:
+                self.assertNotEqual(log.message, "Database changes have been reverted due to error.")
 
     def test_read_only_no_commit_field(self):
         """
@@ -222,10 +244,18 @@ class JobTest(TestCase):
             data = job_class.serialize_data(form.cleaned_data)
 
             # Run the job and extract the job payload data
-            run_job(data=data, request=None, commit=False, job_result_pk=job_result.pk)
+            # Changing commit=True as commit=False will rollback database changes including the
+            # logs that we are trying to read. See above note on why we are using the default database.
+            # Also need to pass a mock request object as execute_webhooks will be called with the creation
+            # of the objects.
+            run_job(data=data, request=self.request, commit=True, job_result_pk=job_result.pk)
             job_result.refresh_from_db()
-            job_payload = job_result.data["run"]["log"][0][2]  # Indexing makes me sad.
-            job_result_data = json.loads(job_payload)
+
+            log_info = JobLogEntry.objects.filter(
+                job_result=job_result, log_level=LogLevelChoices.LOG_INFO, grouping="run"
+            ).first()
+
+            job_result_data = json.loads(log_info.log_object)
 
             # Assert stuff
             self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_COMPLETED)
@@ -256,16 +286,51 @@ class JobTest(TestCase):
             }
 
             # Run the job and extract the job payload data
-            run_job(data=data, request=None, commit=False, job_result_pk=job_result.pk)
+            # See test_ip_address_vars as to why we are changing commit=True and request=self.request.
+            run_job(data=data, request=self.request, commit=True, job_result_pk=job_result.pk)
             job_result.refresh_from_db()
-            job_payload = job_result.data["run"]["log"][0][2]  # Indexing makes me sad.
-            job_result_data = json.loads(job_payload)
+            # Test storing additional data in job
+            job_result_data = job_result.data["object_vars"]
+
+            info_log = JobLogEntry.objects.filter(
+                job_result=job_result, log_level=LogLevelChoices.LOG_INFO, grouping="run"
+            ).first()
 
             # Assert stuff
             self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_COMPLETED)
             self.assertEqual({"role": str(d.pk), "roles": [str(d.pk)]}, job_result_data)
+            self.assertEqual(info_log.log_object, "Role: role")
+            self.assertEqual(job_result.data["output"], "\nNice Roles, bro.")
+
+    def test_job_data_as_string(self):
+        """
+        Test that job doesn't error when not a dictionary.
+        """
+        with self.settings(JOBS_ROOT=os.path.join(settings.BASE_DIR, "extras/tests/dummy_jobs")):
+            module = "test_object_vars"
+            name = "TestObjectVars"
+            job_class = get_job(f"local/{module}/{name}")
+
+            # Prepare the job data
+            job_result = JobResult.objects.create(
+                name=job_class.class_path,
+                obj_type=self.job_content_type,
+                user=None,
+                job_id=uuid.uuid4(),
+            )
+            data = "BAD DATA STRING"
+            run_job(data=data, request=None, commit=False, job_result_pk=job_result.pk)
+            job_result.refresh_from_db()
+
+            # Assert stuff
+            self.assertEqual(job_result.status, JobResultStatusChoices.STATUS_ERRORED)
+            log_failure = JobLogEntry.objects.filter(
+                grouping="initialization", log_level=LogLevelChoices.LOG_FAILURE
+            ).first()
+            self.assertIn("Data should be a dictionary", log_failure.message)
 
 
+@mock.patch("nautobot.extras.models.models.JOB_LOGS", None)
 class JobFileUploadTest(TestCase):
     """Test a job that uploads/deletes files."""
 
@@ -276,8 +341,14 @@ class JobFileUploadTest(TestCase):
         cls.job_content_type = ContentType.objects.get(app_label="extras", model="job")
 
     def setUp(self):
+        super().setUp()
         self.dummy_file.seek(0)  # Reset cursor so we can read it again.
 
+        # Initialize fake request that will be required to execute Webhooks (in jobs.)
+        self.request = RequestFactory().request(SERVER_NAME="WebRequestContext")
+        self.request.id = uuid.uuid4()
+        self.request.user = self.user
+
     def test_run_job_pass(self):
         """Test that file upload succeeds; job SUCCEEDS; and files are deleted."""
         with self.settings(JOBS_ROOT=os.path.join(settings.BASE_DIR, "extras/tests/dummy_jobs")):
@@ -303,13 +374,16 @@ class JobFileUploadTest(TestCase):
             self.assertEqual(FileProxy.objects.count(), 1)
 
             # Run the job
-            run_job(data=serialized_data, request=None, commit=False, job_result_pk=job_result.pk)
+            # See test_ip_address_vars as to why we are changing commit=True and request=self.request.
+            run_job(data=serialized_data, request=self.request, commit=True, job_result_pk=job_result.pk)
             job_result.refresh_from_db()
 
+            warning_log = JobLogEntry.objects.filter(
+                job_result=job_result, log_level=LogLevelChoices.LOG_WARNING, grouping="run"
+            ).first()
+
             # Assert that file contents were correctly read
-            self.assertEqual(
-                job_result.data["run"]["log"][0][2], f"File contents: {self.file_contents}"  # "File contents: ..."
-            )
+            self.assertEqual(warning_log.message, f"File contents: {self.file_contents}")  # "File contents: ..."
 
             # Assert that FileProxy was cleaned up
             self.assertEqual(FileProxy.objects.count(), 0)
@@ -342,17 +416,14 @@ class JobFileUploadTest(TestCase):
             run_job(data=serialized_data, request=None, commit=False, job_result_pk=job_result.pk)
             job_result.refresh_from_db()
 
-            # Assert that file contents were correctly read
-            self.assertEqual(
-                job_result.data["run"]["log"][0][2], f"File contents: {self.file_contents}"  # "File contents: ..."
-            )
-            # Also ensure the standard log message about aborting the transaction is present
-            self.assertEqual(job_result.data["run"]["log"][-1][-1], "Database changes have been reverted due to error.")
+            # Can't check log objects when jobs are reverted (within tests anyways.)
+            # This is due to the fake job_logs db not being available for tests.
 
             # Assert that FileProxy was cleaned up
             self.assertEqual(FileProxy.objects.count(), 0)
 
 
+@mock.patch("nautobot.extras.models.models.JOB_LOGS", None)
 class RunJobManagementCommandTest(CeleryTestCase):
     """Test cases for the `nautobot-server runjob` management command."""
 
@@ -370,10 +441,12 @@ class RunJobManagementCommandTest(CeleryTestCase):
 
     def test_runjob_nochange_successful(self):
         """Basic success-path test for Jobs that don't modify the Nautobot database."""
+        # I had to change this test, unfortunately with no commit, the logs revert since we can't
+        # use the fake job_logs db here.
         with self.settings(JOBS_ROOT=os.path.join(settings.BASE_DIR, "extras/tests/dummy_jobs")):
             out, err = self.run_command("local/test_pass/TestPass")
             self.assertIn("Running local/test_pass/TestPass...", out)
-            self.assertIn("test_pass: 1 success, 1 info, 0 warning, 0 failure", out)
+            self.assertIn("test_pass: 0 success, 1 info, 0 warning, 0 failure", out)
             self.assertIn("info: Database changes have been reverted automatically.", out)
             self.assertIn("local/test_pass/TestPass: SUCCESS", out)
             self.assertEqual("", err)
@@ -383,6 +456,8 @@ class RunJobManagementCommandTest(CeleryTestCase):
         with self.assertRaises(ObjectDoesNotExist):
             Status.objects.get(slug="test-status")
 
+        # I had to change this test, unfortunately with no commit, the logs revert since we can't
+        # use the fake job_logs db here.
         with self.settings(JOBS_ROOT=os.path.join(settings.BASE_DIR, "extras/tests/dummy_jobs")):
             out, err = self.run_command("local/test_modify_db/TestModifyDB")
             self.assertIn("Running local/test_modify_db/TestModifyDB...", out)
@@ -394,6 +469,9 @@ class RunJobManagementCommandTest(CeleryTestCase):
         with self.assertRaises(ObjectDoesNotExist):
             Status.objects.get(slug="test-status")
 
+        info_log = JobLogEntry.objects.filter(log_level=LogLevelChoices.LOG_INFO).first()
+        self.assertEqual("Database changes have been reverted automatically.", info_log.message)
+
     def test_runjob_db_change_commit_no_username(self):
         """A job that changes the DB, when run with commit=True but no username, is rejected."""
         with self.settings(JOBS_ROOT=os.path.join(settings.BASE_DIR, "extras/tests/dummy_jobs")):
@@ -413,10 +491,14 @@ class RunJobManagementCommandTest(CeleryTestCase):
 
             out, err = self.run_command("--commit", "--username", "dummy_user", "local/test_modify_db/TestModifyDB")
             self.assertIn("Running local/test_modify_db/TestModifyDB...", out)
-            self.assertIn("test_modify_db: 0 success, 0 info, 0 warning, 0 failure", out)
+            # Changed job to actually log data. Can't display empty results if no logs were created.
+            self.assertIn("test_modify_db: 1 success, 0 info, 0 warning, 0 failure", out)
             self.assertIn("local/test_modify_db/TestModifyDB: SUCCESS", out)
             self.assertEqual("", err)
 
+        success_log = JobLogEntry.objects.filter(log_level=LogLevelChoices.LOG_SUCCESS).first()
+        self.assertEqual(success_log.message, "Status created successfully.")
+
         status = Status.objects.get(slug="test-status")
         self.assertEqual(status.name, "Test Status")
 
